"use server"

import prisma from "@/lib/prisma";
import { getImageBufferFromS3 } from "@/utils/getImageBufferFromS3";
import { extractColors } from "extract-colors";
import getPixels from "get-pixels";
import { NdArray } from "ndarray";

export async function generateExtractColors(imageId: string, fileKey: string) {
  const image = await prisma.image.findUnique({
    where: {
      id: imageId
    },
    include: {
      metadata: true
    }
  })
  const buffer = await getImageBufferFromS3(fileKey);
  if(!image) throw new Error("Image not found");

  const imageDataUrl = `data:${image.fileType};base64,${buffer.toString("base64")}`;

  const pixels = await new Promise<NdArray<Uint8Array>>((resolve, reject) => {
    getPixels(imageDataUrl, 'image/' + image.metadata[0].format || "image/jpeg", (err, pixels) => {
      if (err) reject(err);
      else resolve(pixels);
    });
  });

  const extracted = await extractColors({
    data: Array.from(pixels.data),
    width: pixels.shape[0],
    height: pixels.shape[1]
  });

  for (const c of extracted) {
    await prisma.extractColor.create({
      data: {
        hex: c.hex,
        red: c.red,
        green: c.green,
        blue: c.blue,
        hue: c.hue,
        saturation: c.saturation,
        area: c.area,
        imageId: imageId,
      },
    });
  }

  return await prisma.extractColor.findMany({
    where: { imageId: imageId }
  });
}