"use server"

import prisma from "@/lib/prisma";
import { VibrantSwatch } from "@/types/VibrantSwatch";
import { getImageBufferFromS3 } from "@/utils/getImageBufferFromS3";
import { rgbToHex } from "@/utils/uploadHelper";
import { Vibrant } from "node-vibrant/node";

export async function generateImageColors(imageId: string, fileKey: string) {
  const buffer = await getImageBufferFromS3(fileKey);
  const palette = await Vibrant.from(buffer).getPalette();

  const vibrantHexes = Object.fromEntries(
    Object.entries(palette).map(([key, swatch]) => {
      const castSwatch = swatch as VibrantSwatch | null;
      const rgb = castSwatch?._rgb;
      const hex = castSwatch?.hex || (rgb ? rgbToHex(rgb) : undefined);
      return [key, hex];
    })
  );
  
  for (const [type, hex] of Object.entries(vibrantHexes)) {
    if (!hex) continue;
    const [r, g, b] = hex.match(/\w\w/g)!.map((h) => parseInt(h, 16));
    await prisma.imageColor.create({
      data: {
        type,
        hex,
        red: r,
        green: g,
        blue: b,
        imageId: imageId,
      },
    });
  }

  return await prisma.imageColor.findMany({
    where: { imageId: imageId }
  });
}